import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * This class is the main drawing area of the program.
 *
 * @author Alberto Manuel B. Simes
 */
public class MapCanvas extends JPanel implements MouseListener, 
						 MouseMotionListener,
						 ActionListener {
    private Map map;
    private boolean editable;
    private boolean zoomed;
    private int imageWidth;
    private int imageHeight;

    private GraphicElement dots;

    /**
     * Constructor for the MapCanvas object.
     *
     * @param map Map object for displaying
     */
    public MapCanvas(Map map) {
	this.map = map;
	this.imageWidth = map.getImage().getWidth(this);
	this.imageHeight = map.getImage().getHeight(this);
	this.addMouseListener(this);
	this.addMouseMotionListener(this);
	this.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
	this.editable = false;
	this.zoomed = false;
	this.dots = null;
	superSetSize();
    }

    public void setEditable(boolean b) {
	this.editable = b;
    }

    public boolean isEditable() {
	return this.editable;
    }

    /**
     * This method will be called whenever we should redraw the canvas.
     *
     * @param graphic A graphic context where the canvas will be
     * drawed
     */
    public void paint(Graphics g) {
	if (zoomed) {
	    g.drawImage(map.getImage(),0,0,2*imageWidth,2*imageHeight,this);
	} else {
	    g.drawImage(map.getImage(),0,0,this);
	}
	
	Enumeration layers = map.getLayers().elements();
	while (layers.hasMoreElements()) {
	    Layer l = (Layer)layers.nextElement();
	    l.show(g, zoomed);
	}


	if (dots != null) {
	    dots.show(g, zoomed, new Color(0, 0, 255));
	}
    }

    /**
     *
     */
    public void mouseDragged(MouseEvent e) {
	if (dots!=null) {
	    CanvasDot dot = null;
	    for (Enumeration enum = dots.elements(); enum.hasMoreElements() && dot == null; ) {
		CanvasDot pt = (CanvasDot)enum.nextElement();
		if (pt.isActive()) dot = pt;
	    }
	    if (dot!=null) {
		Point currentPoint = e.getPoint();
		if (zoomed) currentPoint = new Point((int)currentPoint.getX()/2,
						     (int)currentPoint.getY()/2);
		dot.setLocation(currentPoint);
		repaint();
	    }
	}
    }

    /**
     *
     */
    public void mouseMoved(MouseEvent e) {
	if (dots != null) {
	    Point currentPoint = e.getPoint();
	    if (zoomed) currentPoint = new CanvasDot((int)currentPoint.getX()/2,
						     (int)currentPoint.getY()/2,true);

	    boolean needsRedraw = false;
	    Enumeration each = dots.elements();
	    while (each.hasMoreElements()) {
		CanvasDot pt = (CanvasDot)each.nextElement();
		if (pt.distance(currentPoint) < 5) {
		    if (!pt.isActive()) {
			pt.setActive(true);
			needsRedraw = true;
		    }
		} else {
		    if (pt.isActive()) {
			pt.setActive(false);
			needsRedraw = true;
		    }
		}
	    }
	    if (needsRedraw) repaint();
	}
    }

    public void setCurrentObject(GraphicElement gel) {
	this.dots = gel;
    }
    public GraphicElement getCurrentObject() {
	return this.dots;
    }

    /**
     *
     */
    public void setSize(int width, int height) {}

    /**
     *
     */
    public void setSize(Dimension d) {}

    /**
     *
     */
    public Dimension getSize()          { 
	if (zoomed) {
	    return new Dimension(2*imageWidth, 2*imageHeight);
	} else {
	    return new Dimension(imageWidth, imageHeight); 
	}
    }

    /**
     *
     */
    public Dimension getPreferredSize() { return getSize(); }
    /*
     *
     */
    public Dimension getMinimumSize()   { return getSize(); }
    /*
     *
     */
    public Dimension getMaximumSize()   { return getSize(); }

    // MouseListener implementation
    /*
     *
     */
    public void mouseClicked(MouseEvent e) {
	CanvasDot currentDot;
	if (editable) {
	    System.out.println("Clicked");
	}

	currentDot = new CanvasDot(e.getPoint(),true);
	if (zoomed) currentDot = new CanvasDot((int)currentDot.getX()/2,
					       (int)currentDot.getY()/2,true);
	if (dots!=null) {
	    dots.add(currentDot);
	    this.repaint();
	}
    }
    /*
     *
     */
    public void mouseEntered(MouseEvent e)  {    }
    /*
     *
     */
    public void mouseExited(MouseEvent e)   {    }
    /*
     *
     */
    public void mousePressed(MouseEvent e)  {    }
    /*
     *
     */
    public void mouseReleased(MouseEvent e) {    }

    // ActionListener implementation
    /*
     *
     */
    public void actionPerformed(ActionEvent e) {
	if (e.getActionCommand().equals(" Zoom In ")) 
	{
	    JButton bt = (JButton)e.getSource();
	    zoomed = true;
	    bt.setText(" Zoom Out ");
	    firePropertyChange(new String("zoom"), !zoomed, zoomed);
	    superSetSize();
	} 
	else
	    if (e.getActionCommand().equals(" Zoom Out ")) 
	    {
		JButton bt = (JButton)e.getSource();
		zoomed = false;
		bt.setText(" Zoom In ");
		firePropertyChange(new String("zoom"), !zoomed, zoomed);
		superSetSize();
	    } 
	    else 
	    {
		System.out.println("MapCanvas actionPerformed function");
		System.out.println(e.toString());
	    }
    }

    /*
     *
     */
    private void superSetSize() {
	if (zoomed) {
	    super.setSize(2 * imageWidth, 2 * imageHeight);
	} else {
	    super.setSize(imageWidth, imageHeight);
	}
    }

    private void drawPoint(Graphics g, CanvasDot p) {
	// Set color for the dot and draw it.
	g.setColor( p.isActive() ? new Color(255,0,0) : new Color(0,0,255) );
	// Check if we are zoomed...
	if (zoomed) {
	    g.drawRoundRect((int)(2*p.getX()-12), (int)(2*p.getY()-12), 24,24,8,8);
	    g.drawRect((int)(2*p.getX()-6), (int)(2*p.getY()-6), 12, 12);
	} else {
	    g.drawRoundRect((int)p.getX()-6, (int)p.getY()-6, 12,12,4,4);
	    g.drawRect((int)p.getX()-3, (int)p.getY()-3, 6, 6);
	}
    }

    private void drawLine(Graphics g, CanvasDot first, CanvasDot second) {
	g.setColor( new Color(0,0,255) );
	if (zoomed) {
	    g.drawLine((int)(2*first.getX()), (int)(2*first.getY()),
		       (int)(2*second.getX()), (int)(2*second.getY()));
	} else {
	    g.drawLine((int)first.getX(), (int)first.getY(),
		       (int)second.getX(), (int)second.getY());
	}
    }

}
